#include <Adafruit_SPIFlash.h>
#include <nrf52840.h>

#define DO_WORK_PIN   D2
#define SHUTDOWN_PIN  D3

Adafruit_FlashTransport_QSPI flashTransport;

enum commands_t
{
    DA_CMD_DO_NOTHING = 0,
    DA_CMD_DO_WORK,
    DA_CMD_SYSTEM_OFF_SLEEP
};
static int da_command = DA_CMD_DO_NOTHING;
static SemaphoreHandle_t da_xSemaphore;
static int da_work_LED_status = HIGH;

void QSPIF_sleep(void)
{
  flashTransport.begin();
  flashTransport.runCommand(0xB9);
  flashTransport.end();
}

void setup()
{
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  da_work_LED_status = HIGH;
  digitalWrite(LED_BLUE, da_work_LED_status);

  //start bluefruit (this also starts the soft device if you don't start the sd then none of your sd_* calls will do anything
  //Bluefruit.begin();

  QSPIF_sleep();

  da_xSemaphore = xSemaphoreCreateBinary();
  da_command = DA_CMD_DO_NOTHING;

  pinMode(DO_WORK_PIN, INPUT_PULLUP_SENSE);
  attachInterrupt(digitalPinToInterrupt(DO_WORK_PIN), doWorkISR, FALLING);

  pinMode(SHUTDOWN_PIN, INPUT_PULLUP_SENSE);
  attachInterrupt(digitalPinToInterrupt(SHUTDOWN_PIN), shutdownISR, FALLING);

  // Flash green to see power on, reset, and wake from system_off
  digitalWrite(LED_GREEN, LOW);
  delay(1000);
  digitalWrite(LED_GREEN, HIGH);;

  // Minimizes power when bluetooth is used
  NRF_POWER->DCDCEN = 1;
}

void doWorkISR()
{
  xSemaphoreGive(da_xSemaphore);
  da_command = DA_CMD_DO_WORK;
}

void shutdownISR()
{
  da_command = DA_CMD_SYSTEM_OFF_SLEEP;
  xSemaphoreGive(da_xSemaphore);
}

void loop()
{
  // FreeRTOS will automatically put the system in system_on sleep mode here
  xSemaphoreTake(da_xSemaphore, portMAX_DELAY);

  // Something caused the system to wake up

  if (da_command == DA_CMD_DO_WORK)
  {
    // Do work
    da_work_LED_status = !da_work_LED_status;
    digitalWrite(LED_BLUE, da_work_LED_status);
  }
  else if (da_command == DA_CMD_SYSTEM_OFF_SLEEP)
  {
    //Flash red to see we are going to system_off sleep mode
    digitalWrite(LED_RED, LOW);
    delay(1000);
    digitalWrite(LED_RED, HIGH);

    NRF_POWER->SYSTEMOFF=1; // Execution should not go beyond this
    //sd_power_system_off(); // Use this instead if using the soft device
  }
}